function split = findsplit(X,y)

global priors;
global Nj;
global nC;


nclasses = nC;
N = size(X,1);

critfun = @gdi;

nvars=size(X,2);
Nnode=size(X,1);
Cnode = false(Nnode,nclasses);
Cnode(sub2ind([N nclasses],(1:N)',y)) = 1;

Njt = sum(Cnode,1);    % number in class j at node t
Pjandt = priors .* Njt ./ Nj;
Pt = sum(Pjandt); % == length(y)/Nnode;
Pjgivent = Pjandt / Pt;
%misclasscost = Pjgivent * Cost;
%[mincost,nodeclass] = min(misclasscost);
%yfitnode(tnode) = nodeclass;
pratio = priors ./ Nj;
%impure = sum(Pjgivent>0)>1;

bestcrit = -Inf;


for jvar=1:nvars

    [x,idx] = sort(X(:,jvar));          % get sorted jth x variable

    % Determine if there's anything to split along this variable
    maxeps = max(eps(x(1)), eps(x(end)));
    if x(1)+maxeps > x(end)
        continue;
    end
    rows = find(x(1:end-1)+maxeps < x(2:end));
    if isempty(rows)
        continue; % pass control to the next iteration of the for loop or while loop
    end
    
    Ccum = cumsum(Cnode(idx,:)); % cum. class counts
    %[critval,cutval]=Ccritval(x,Ccum,rows,xcat,pratio,Pt,impurity,critfun,bestcrit);
    [critval,cutval]=Ccritval(x,Ccum,rows,pratio,Pt,bestcrit,critfun);
    
    if critval>bestcrit
        bestcrit = critval;
        bestvar = jvar;
        bestcut = cutval;
    end
    
end

split.infgain = 0;%infgain;
split.critval = bestcrit;%infgain;
split.varid = bestvar;
split.splitval = bestcut;


%----------------------------------------------------
function v=gdi(p)
%GDI Gini diversity index

v=1-sum(p.^2,2);

%----------------------------------------------------
function [critval,cutval]=Ccritval(x,Ccum,rows,pratio,Pt,bestcrit,critfun)
%CCRITVAL Get critical value for splitting node in classification tree.
    
% First get all possible split points
critval = -Inf;
cutval = 0;

% Get arrays showing left/right class membership at each split
nsplits = length(rows);

% Split between each pair of distinct ordered values
Csplit1 = Ccum(rows,:);
Csplit2 = Ccum(size(Ccum,1)*ones(nsplits,1),:) - Csplit1; % repmat(Ccum(end,:),nsplits,1) - Csplit1;
    
% Get left/right class probabilities at each split
temp = pratio(ones(nsplits,1),:); %repmat(pratio,nsplits,1);
P1 = temp .* Csplit1;
P2 = temp .* Csplit2;
Ptleft  = sum(P1,2);
Ptright = sum(P2,2);
nclasses = size(P1,2);
wuns = ones(1,nclasses);
P1 = P1 ./ Ptleft(:,wuns);   %repmat(Ptleft,1,nclasses);
P2 = P2 ./ Ptright(:,wuns);  %repmat(Ptright,1,nclasses);

% Get left/right node probabilities
Pleft = Ptleft ./ Pt;
Pright = 1 - Pleft;

crit = - Pleft.*feval(critfun,P1);
t = (crit>bestcrit);   % compute 2nd term only if it would make a difference
if any(t)
    crit(t) = crit(t) - Pright(t).*feval(critfun,P2(t,:));
end

% Return best split point, but bail out early if no improvement
critval = max(crit);
if critval<bestcrit
   cutval = 0;
   return;
end

maxloc = find(crit==critval);
if length(maxloc)>1
   maxloc = maxloc(1+floor(length(maxloc)*rand));
end

cutloc = rows(maxloc);
cutval = (x(cutloc) + x(cutloc+1))/2;
