function T = grow_tree(X,y)

%disp(' ');

% number of variables in X (dimension)
nd=size(X,2);
disp([' Number of variables in X: ',num2str(nd)]);

% number of classes in y
mcl = max(y);
nclasses = mcl;
disp([' Max class label in y: ',num2str(mcl)]);


% INIT VARIABLES USED FOR TREE CONSTRUCTON
maxdepth = round(realmax); %max tree depth
tol = 0.0; % error tolerance to accept as leaf
minnum = 10; % minimal number at node to split
global N;
    N=size(X,1); % total number of data points in X
global nC;
    nC=mcl; % number of classes in y
global Nj;
    C = false(N,nclasses);
    C(sub2ind([N nclasses],(1:N)',y)) = 1;
    Nj = sum(C,1);
    %Nj=[];
    %for j=1:nC
    %    Nj=[Nj,sum(y==j)];
    %end  
global priors; % prior class propbabilities
    priors = Nj / N;
    %priors=[]; % calculate default priors, data estimated
    %for j=1:nC
    %    priors = [priors,Nj(j)/length(y)];
    %end
havecosts = 1;
global Cost; % cost matrix C(i|j)
    Cost=ones(mcl,mcl)-eye(mcl,mcl); % default (unit) cost matrix, equal misclassification cost
    %havecosts = false;
    % this case R(T) is prob of misclassification, otherwise R(T) is
    % misclassification cost
    
%Cost=Cost*2;Cost(1,2)=1;Cost
Cost(1,2)=1;Cost    

% Alter priors according to Cost, page 114
if havecosts
    Cj = sum(Cost',2)';
    pc = Cj .* priors;
    priors = pc / sum(pc); 
end
priors    

% check if X is leaf
[clabel,islf,p,r,pjtc] = nodedata(y,tol,minnum);
if (islf)
    disp('Cannot create tree, initial node is leaf!');
    return;
end

T=[];
T = grow_tree_recursion(T,X,y,1,maxdepth,tol,minnum);
T(1).priors=priors;

T = removebadleafs(T);

%-----------------------------------------------
function T = grow_tree_recursion(T,X,y,curdepth,maxdepth,tol,minnum)

global N; % number of original data points
global nC; % number of original classes
global priors; % prior class probabilities
global Cost; % cost matrix C(i|j)

% next tree node position
position = length(T)+1;

if (position == 29)
%disp(''); 
end

T(position).std=std(X);
T(position).mean=mean(X);

split = findsplit(X,y);
T(position).split = split;

T(position).depth = curdepth;

[clabel,islf,p,r,pjtc] = nodedata(y,tol,minnum);
T(position).p=p; % prob that a certain case falls into this node; = sum(length(y)/N);
T(position).r=r; % resub error prob at current node
T(position).classlabel = clabel; % most probable class label

[Xsmall,Xbig,ysmall,ybig] = divide_x_using_split(X,y,split);

%compute statistics for distance meaurement and add data to note childs
Xsmall_mean=mean(Xsmall);
if ( length(ysmall)-1 )
    Xsmall_cov=Xsmall'*Xsmall/(length(ysmall)-1)-Xsmall_mean'*Xsmall_mean;
else
    Xsmall_cov=eye(nC,nC);
end
Xbig_mean=mean(Xbig);
if ( length(ybig)-1 )
    Xbig_cov=Xbig'*Xbig/(length(ybig)-1)-Xbig_mean'*Xbig_mean;
else
    Xbig_cov=eye(nC,nC);
end
T(position).smallermean=Xsmall_mean;
T(position).biggermean=Xbig_mean;
T(position).smallercov=Xsmall_cov;
T(position).biggercov=Xbig_cov;

if (size(Xbig_mean,1) ~= 1)
    disp('');
end
if (size(Xsmall_mean,1) ~= 1)
    disp('');
end
    
%if ( (size(Xsmall,1) == size(X,1)) | (size(Xbig,1) == size(X,1)) )
%	T(position).smallerchild = -1;
%	T(position).biggerchild = -1;
%	T(position).depth = curdepth;
%	return
%elseif (curdepth == maxdepth) 
if (curdepth == maxdepth) 
	% add smaller child data
    [clabel,islf,p,r,pjtc] = nodedata(ysmall,tol,minnum);
    T(position).smallerchild = -1; % mark as a leaf
    T(position).sclasslabel = clabel; % most probable class label
    T(position).sp=p; % prob that a certain case falls into this node
    T(position).sr=r; % resub error prob at current node
    T(position).spjtc=pjtc;
	% add bigger child data
    [clabel,islf,p,r,pjtc] = nodedata(ybig,tol,minnum);
    T(position).biggerchild = -1; % mark as a leaf
    T(position).bclasslabel = clabel; % most probable class label
    T(position).bp=p; % prob that a certain case falls into this node
    T(position).br=r; % resub error prob at current node
    T(position).bpjtc=pjtc;
	
	return    
else% curdepth ~= maxdepth
    % determine if the left (smaller) child will be a leaf:
	[clabel,islf,p,r,pjtc] = nodedata(ysmall,tol,minnum);
    if (islf)
        % add smaller child data
		T(position).smallerchild = -1; % mark as a leaf
        T(position).sclasslabel = clabel; % most probable class label
        T(position).sp=p; % prob that a certain case falls into this node
        T(position).sr=r; % resub error prob at current node
        T(position).spjtc=pjtc;
	else
		T(position).smallerchild = length(T) + 1;	% next node position
        T(position).spjtc=pjtc;
		T = grow_tree_recursion(T,Xsmall,ysmall,curdepth+1,maxdepth,tol,minnum);
    end
	% determine if the right (bigger) child will be a leaf:
	[clabel,islf,p,r,pjtc] = nodedata(ybig,tol,minnum);
    if (islf)
        % add bigger child data
		T(position).biggerchild = -1; % mark as a leaf
        T(position).bclasslabel = clabel; % most probable class label
        T(position).bp=p; % prob that a certain case falls into this node
        T(position).br=r; % resub error prob at current node
        T(position).bpjtc=pjtc;
	else
		T(position).biggerchild = length(T) + 1;	% next node position
        T(position).bpjtc=pjtc;
		T = grow_tree_recursion(T,Xbig,ybig,curdepth+1,maxdepth,tol,minnum);
    end
end % if

return

%-------------------------------------
function rT = removebadleafs(T)

for k=1:length(T)
    for pos=1:length(T)
        if (T(pos).smallerchild == -1) & (T(pos).biggerchild == -1)
            if (T(pos).sclasslabel) == (T(pos).bclasslabel)
                
                aid = 0;
                for i=1:length(T)
                    if (T(i).smallerchild == pos)
                        aid=-1*i;
                    end
                    if (T(i).biggerchild == pos)
                        aid=i;
                    end
                end

                if (aid < 0)
                    T(abs(aid)).smallerchild = -1;
                    T(abs(aid)).sclasslabel = T(pos).classlabel;
                    T(abs(aid)).sp = T(pos).p;
                    T(abs(aid)).sr = T(pos).r;
                elseif (aid > 0)
                    T(aid).biggerchild = -1;
                    T(aid).bclasslabel = T(pos).classlabel;
                    T(aid).bp = T(pos).p;
                    T(aid).br = T(pos).r;
                else
                    %disp('Node not found!'); % ezt ki kell javitani,
                    %sokszor lefut feleslegesen
                end
                
            end
        end
    end
    
    rT = branch(T,1);
end